// Program Headers
#include "rom_associate.h"
#include "qtvba.h"

// QT Headers
#include <qdir.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qprogressdialog.h>
#include <qcheckbox.h>
//#include <qhttp.h>
#include <qnetwork.h>
#include <qurloperator.h>
#include <qprocess.h>

// c++ headers
#include <iostream.h>

// images

rom_associate::rom_associate ( QString temppath, QString templisting, QString vba_path_temp, QWidget* parent,  const char* name, WFlags fl ) : rom_associate_parent ( parent, name, fl ) {
	vba_path = vba_path_temp;
	path = temppath;
	listing = templisting;
	filter_directory();
	filter_index();
	// pushButton17->setDefault(true);

}

void rom_associate::associate() {
	if (rom_directory_listbox->selectedItem() != 0 && rom_index_listbox->selectedItem() != 0) {
		QString orig_filename = path + QString("/") + rom_directory_listbox->selectedItem()->text();
		QString  new_filename = path + QString("/") + rom_index_listbox->selectedItem()->text();
		QFile qf (new_filename);
		if (qf.exists()) {
			if (QMessageBox::warning(this, "File Exists Error", "The destination filename " + new_filename + " already exists!", "Cancel", "Overwrite")) {
				if (!(new QDir())->rename(orig_filename, new_filename)) {
					QMessageBox::warning(this, "Rename Error", "There was an error renaming the file", "Bugger!");
				} else {
					rom_directory_listbox->removeItem(rom_directory_listbox->currentItem());
					rom_directory_listbox->clearSelection();
					rom_index_listbox->clearSelection();
				}
			}
		} else { 
			if (!(new QDir())->rename(orig_filename, new_filename)) {
				QMessageBox::warning(this, "startwarn", "There was an error renaming the file", "Bugger!");
			} else {
				rom_directory_listbox->removeItem(rom_directory_listbox->currentItem());
				rom_directory_listbox->clearSelection();
				rom_index_listbox->clearSelection();
			}
		}
	// OKAY Lets repeat that for save games.
		for (int i = 1; i < 11; i++) {
			QString save_orig_filename = orig_filename.mid(0, orig_filename.length()-4) + QString::number(i) + QString(".sgm");
			QString save_new_filename = new_filename.mid(0, new_filename.length()-4) + QString::number(i) + QString(".sgm"); 
			QFile save_orig_qf (save_orig_filename);
			QFile save_new_qf (save_new_filename);

			if (save_orig_qf.exists()) {
				if (save_new_qf.exists()) {
					if (QMessageBox::warning(this, "File Exists Error", "The destination filename" + save_new_filename + " already exists!", "Cancel", "Overwrite")) {
						if (!(new QDir())->rename(save_orig_filename, save_new_filename)) {
							QMessageBox::warning(this, "Rename Error", "There was an error renaming the file " + save_orig_filename , "OK");
						}
					}
				} else {
					if (!(new QDir())->rename(save_orig_filename, save_new_filename)) {
					   QMessageBox::warning(this, "Rename Error", "There was an error renaming the file " + save_orig_filename , "OK");
					}
				}
			}
		}

	}
	emit(rom_renamed());
}

QStringList rom_associate::getAssociations() {
	return associations;
}

void rom_associate::filter_index() {
	rom_index_listbox->clear();
	QString filename = QDir::homeDirPath() + "/.qtvba/List.txt";
	QStringList lines;
	QFile file( filename );
	        if ( file.open( IO_ReadOnly ) ) {
		QTextStream stream( &file );
		QString line;
		while ( !stream.atEnd() ) {
			line = stream.readLine(); 
			lines.append(line.latin1() + QString(".gba"));

		}
		file.close();
	}
	if (index_filter_le->text() == "") {
		rom_index_listbox->insertStringList(lines);
	} else {
		rom_index_listbox->insertStringList(lines.grep(index_filter_le->text(), false));
	}
}

void rom_associate::filter_directory() {
	rom_directory_listbox->clear();
	QDir romdir(path);
	romdir.setFilter(QDir::Files | QDir::Readable);
	if (directory_filter_le->text() == "") {
		rom_directory_listbox->insertStringList(romdir.entryList("*.GBA *.gba *.bin"));
	} else {
		rom_directory_listbox->insertStringList(romdir.entryList("*.GBA *.gba *.bin").grep(directory_filter_le->text(), false));
	}
}

void rom_associate::remove_matching_filenames() {
	if (remove_matching_filenames_cb->isChecked() == true) {
		index_filter_le->clear();
		filter_index();
		QListBoxItem * qlbi = rom_directory_listbox->firstItem(), * index_qlbi; 
		while (qlbi != NULL) {
			index_qlbi = rom_index_listbox->findItem(qlbi->text());
			if (index_qlbi != NULL) {
				rom_directory_listbox->removeItem(rom_directory_listbox->index(qlbi));
			}
			qlbi = qlbi->next();
		}
	} else {
		filter_directory();
	}
}

void rom_associate::dl_new_listing() {
	qInitNetworkProtocols();
	QUrlOperator *op = new QUrlOperator();
	QString destination = QDir::homeDirPath() + QString("/.qtvba/"); 
	QProgressDialog progress( "Downloading listing...", "Cancel", 100);
	connect(op, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * )), &progress, SLOT(setProgress(int, int)));
	op->copy( listing, destination);
	progress.exec();
	op->stop();
}

void rom_associate::play_rom() {
	if (rom_directory_listbox->selectedItem() != NULL) {
		QString rom_to_play=rom_directory_listbox->selectedItem()->text();
		QProcess * proc = new QProcess (this);
		proc->addArgument(vba_path);
		proc->addArgument(path + QString("/") + rom_to_play);
		if (!proc->start()) 
			QMessageBox::warning(this, "VisualBoyAdvance Launch Error", "There was an error executing VisualBoyAdvance", "OK");
	
	}
}
